## make R plots and summary stats for several simulation settings
## works good with 2 x 2 different settings


## run this file for a whole directory with simulation output, use
## ./scripts/statsplots.pl


## with current color settings, there should not be more than 8 runsets
##    without a 'neutral' run (colors will be recycled)
##---------------------------------------------------------------------


library(RColorBrewer)


myfiles<-commandArgs(trailingOnly=T)[1] ## infile names

mydir<-"." ## chdir done with Perl already


##-------------------
## READ & SET UP ####
##-------------------

runnames<-as.vector(read.table(paste(mydir, "/",myfiles,sep=""),
                               colClasses="character"))

i<-1
tmp<-read.csv(paste(mydir, "/",runnames[i],".stats",sep="")) ## C stats output

mystats<-array(dim=c(dim(tmp),length(runnames)),
               dimnames=list(NULL,colnames(tmp),runnames))
mystats[,,i]<-as.matrix(tmp)
if(length(runnames)>1){
    for(i in 2:length(runnames)){
        tmp<-read.csv(paste(mydir, "/",runnames[i],".stats",sep=""))
        mystats[,,i]<-as.matrix(tmp)
    }
}

neutr<-grep("neutral",runnames)
nsets<-length(neutr)
if(nsets>0){
    nruns<-length(runnames)/nsets
} else {
    nruns<-length(runnames)
    nsets<-1
}

## nruns colors, repeat nsets times, neutral run in black
runcols<-character(nruns)
if(length(neutr)>0){
    runcols[-neutr[1]]<-brewer.pal(n=8,name="Dark2")[1:nruns][-neutr[1]]
    runcols[neutr[1]]<-"black"
    runcols<-rep(runcols,nsets)
} else {
    runcols<-brewer.pal(n=8,name="Dark2")[1:nruns]
}

nrep<-max(as.numeric(mystats[,1,1]))
ngen<-max(as.numeric(mystats[,2,1]))
ndeme<-length(unique(mystats[,3,1]))-1

## ----------------------------
## PLOTS summarizing replicates
## ----------------------------

pdf(paste(mydir, "/",myfiles,"_stats",".pdf",sep=""))

## -----------------
## het and admixture
## -----------------

## mean and sd for q and het over replicates
## -----------------------------------------

## admixture
par(mfrow=c(3,nsets),mar=c(3.5,4,1.5,1))

for(d in 1:ndeme){

    for(k in 1:length(runnames)){

        meanmeanq<-numeric()
        sdmeanq<-numeric() ## sd of mean q of reps
        meansdq<-numeric() ## mean of sd q of reps

        for(g in 0:ngen){ ## stats file includes generation 0
            myrow<-seq(d+g*(ndeme+1),dim(mystats)[1],(ndeme+1)*(ngen+1))

            meanmeanq<-c(meanmeanq,mean(as.numeric(mystats[myrow,5,k]),na.rm=T))
            sdmeanq<-c(sdmeanq,sd(as.numeric(mystats[myrow,5,k]),na.rm=T))
            meansdq<-c(meansdq,mean(as.numeric(mystats[myrow,6,k]),na.rm=T))
        }

        if(!(k-1)%%nruns){
            plot(0:ngen,rep(0.5,ngen+1),ylim=c(-0.2,1.2),axes=F,xlab="",ylab="",type="n")

            box()
            axis(1,at=seq(0,ngen,10),padj=-0.4)
            axis(2,at=c(0,0.25,0.5,0.75,1),padj=0.6)
            if (d%%3==0){mtext("Generation",side=1,line=2,cex=1)}
            mtext("Admixture prop. q",side=2,line=2,cex=1)
            mtext(paste("Deme",d,sep=" "),side=3,line=0,cex=0.9)

            if(d%%3==0){
                legend("topleft",legend=c("mean q","+/- sd for mean q","mean of sd q"),
                       col="darkgray",lwd=c(2,0.8,2), lty=c(1,1,2), bty="n")
            }
            if((d+1)%%3==0){
                legend("topleft",legend=runnames[k:(k+nruns-1)],col=runcols[1:nruns],
                       lwd=1.2, lty=1, bty="n")
            }
        }

        lines(0:ngen,meanmeanq,col=runcols[k],lty=1,lwd=2)
        lines(0:ngen,meanmeanq-sdmeanq,col=runcols[k],lty=1,lwd=0.8)
        lines(0:ngen,meanmeanq+sdmeanq,col=runcols[k],lty=1,lwd=0.8)
        lines(0:ngen,meansdq,col=runcols[k],lty=2,lwd=2)

    }

}

## heterozygosity
par(mfrow=c(3,nsets),mar=c(3.5,4,1.5,1))

for(d in 1:ndeme){

    for(k in 1:length(runnames)){

        meanmeanhet<-numeric()
        sdmeanhet<-numeric() ## sd of mean het of reps
        meansdhet<-numeric() ## mean of sd het of reps

        for(g in 0:ngen){ ## stats file includes generation 0
            myrow<-seq(d+g*(ndeme+1),dim(mystats)[1],(ndeme+1)*(ngen+1))

            meanmeanhet<-c(meanmeanhet,mean(as.numeric(mystats[myrow,7,k]),na.rm=T))
            sdmeanhet<-c(sdmeanhet,sd(as.numeric(mystats[myrow,7,k]),na.rm=T))
            meansdhet<-c(meansdhet,mean(as.numeric(mystats[myrow,8,k]),na.rm=T))
        }

        if(!(k-1)%%nruns){
            plot(0:ngen,rep(0.5,ngen+1),ylim=c(-0.2,1.2),axes=F,xlab="",ylab="",type="n")

            box()
            axis(1,at=seq(0,ngen,10),padj=-0.4)
            axis(2,at=c(0,0.25,0.5,0.75,1),padj=0.6)
            if (d%%3==0){mtext("Generation",side=1,line=2,cex=1)}
            mtext("Inter-source anc.",side=2,line=2,cex=1)
            mtext(paste("Deme",d,sep=" "),side=3,line=0,cex=0.9)

            if(d%%3==0){
                legend("topleft",legend=c("mean het","+/- sd for mean het",
                                     "mean of sd het"),col="darkgray",
                       lwd=c(2,0.8,2), lty=c(1,1,2), bty="n")
            }
            if((d+1)%%3==0){
                legend("topleft",legend=runnames[k:(k+nruns-1)],col=runcols[1:nruns],
                       lwd=1.2, lty=1, bty="n")
            }
        }

        lines(0:ngen,meanmeanhet,col=runcols[k],lty=1,lwd=2)
        lines(0:ngen,meanmeanhet-sdmeanhet,col=runcols[k],lty=1,lwd=0.8)
        lines(0:ngen,meanmeanhet+sdmeanhet,col=runcols[k],lty=1,lwd=0.8)
        lines(0:ngen,meansdhet,col=runcols[k],lty=2,lwd=2)

    }

}


## -----------
## fis and fst
## -----------

par(mfrow=c(2,nsets),mar=c(3.5,4,1.5,1))


for(k in 1:length(runnames)){

    meanfst<-numeric()
    sdfst<-numeric()

    for(g in 0:ngen){ ## stats file includes generation 0
        myrow<-seq((ndeme+1)+g*(ndeme+1),dim(mystats)[1],(ndeme+1)*(ngen+1)) ## deme "meta"

        meanfst<-c(meanfst,mean(as.numeric(mystats[myrow,15,k]),na.rm=T))
        sdfst<-c(sdfst,sd(as.numeric(mystats[myrow,15,k]),na.rm=T))
    }

    if(!(k-1)%%nruns){
        plot(0:ngen,rep(0.5,ngen+1),ylim=c(-0.2,1.2),axes=F,xlab="",ylab="",type="n")

        box()
        axis(1,padj=-0.4)
        axis(2,at=c(0,0.25,0.5,0.75,1),padj=0.6)

        mtext(expression('meta F'['ST']),side=2,line=2,cex=1)

        legend("topright",legend=c("mean","+/- sd"),col="darkgray",
               lwd=c(2,0.8), lty=c(1,1), bty="n")
    }

    lines(0:ngen,meanfst,col=runcols[k],lty=1,lwd=2)
    lines(0:ngen,meanfst-sdfst,col=runcols[k],lty=1,lwd=0.8)
    lines(0:ngen,meanfst+sdfst,col=runcols[k],lty=1,lwd=0.8)

}

for(k in 1:length(runnames)){

    meanfis<-numeric()
    sdfis<-numeric()

    for(g in 0:ngen){ ## stats file includes generation 0
        myrow<-seq((ndeme+1)+g*(ndeme+1),dim(mystats)[1],(ndeme+1)*(ngen+1)) ## deme "meta"

        meanfis<-c(meanfis,mean(as.numeric(mystats[myrow,14,k]),na.rm=T))
        sdfis<-c(sdfis,sd(as.numeric(mystats[myrow,14,k]),na.rm=T))
    }

    if(!(k-1)%%nruns){
        plot(0:ngen,rep(0.5,ngen+1),ylim=c(-0.2,1.2),axes=F,xlab="",ylab="",type="n")

        box()
        axis(1,padj=-0.4)
        axis(2,at=c(0,0.25,0.5,0.75,1),padj=0.6)

        mtext(expression('meta F'['IS']),side=2,line=2,cex=1)
        mtext("Generation",side=1,line=2,cex=1)

        legend("topright",legend=runnames[k:(k+nruns-1)],col=runcols[1:nruns],
               lwd=1.2, lty=1, bty="n")

    }

    lines(0:ngen,meanfis,col=runcols[k],lty=1,lwd=2)
    lines(0:ngen,meanfis-sdfis,col=runcols[k],lty=1,lwd=0.8)
    lines(0:ngen,meanfis+sdfis,col=runcols[k],lty=1,lwd=0.8)

}


## --------------------------------------------------
## mean and sd for fitness and njunct over replicates
## --------------------------------------------------

## fitness
par(mfrow=c(3,nsets),mar=c(3.5,4,1.5,1))

for(d in 1:ndeme){

    for(k in 1:length(runnames)){

        meanmeanfitness<-numeric()
        sdmeanfitness<-numeric() ## sd of mean fitness of reps
        meansdfitness<-numeric() ## mean of sd fitness of reps

        for(g in 0:ngen){ ## stats file includes generation 0
            myrow<-seq(d+g*(ndeme+1),dim(mystats)[1],(ndeme+1)*(ngen+1))

            meanmeanfitness<-c(meanmeanfitness,mean(as.numeric(mystats[myrow,9,k]),na.rm=T))
            sdmeanfitness<-c(sdmeanfitness,sd(as.numeric(mystats[myrow,9,k]),na.rm=T))
            meansdfitness<-c(meansdfitness,mean(as.numeric(mystats[myrow,10,k]),na.rm=T))
        }

        if(!(k-1)%%nruns){
            plot(0:ngen,rep(0.5,ngen+1),ylim=c(-0.2,1.2),axes=F,xlab="",ylab="",type="n")

            box()
            axis(1,at=seq(0,ngen,10),padj=-0.4)
            axis(2,at=c(0,0.25,0.5,0.75,1),padj=0.6)
            if (d%%3==0){mtext("Generation",side=1,line=2,cex=1)}
            mtext("Fitness",side=2,line=2,cex=1)
            mtext(paste("Deme",d,sep=" "),side=3,line=0,cex=0.9)

            if(d%%3==0){
                legend("left",legend=c("mean fitness","+/- sd for mean fitness",
                                  "mean of sd fitness"),
                       col="darkgray", lwd=c(2,0.8,2), lty=c(1,1,2), bty="n")
            }
            if((d+2)%%3==0){
                legend("left",legend=runnames[k:(k+nruns-1)],col=runcols[1:nruns],
                       lwd=1.2, lty=1, bty="n")
            }
        }

        lines(0:ngen,meanmeanfitness,col=runcols[k],lty=1,lwd=2)
        lines(0:ngen,meanmeanfitness-sdmeanfitness,col=runcols[k],lty=1,lwd=0.8)
        lines(0:ngen,meanmeanfitness+sdmeanfitness,col=runcols[k],lty=1,lwd=0.8)
        lines(0:ngen,meansdfitness,col=runcols[k],lty=2,lwd=2)

    }

}

## number of junctions
maxnjunct<-max(as.numeric(mystats[,11,]),na.rm=T)

par(mfrow=c(3,nsets),mar=c(3.5,4,1.5,1))

for(d in 1:ndeme){

    for(k in 1:length(runnames)){

        meanmeannjunct<-numeric()
        sdmeannjunct<-numeric() ## sd of mean njunct of reps
        meansdnjunct<-numeric() ## mean of sd njunct of reps

        for(g in 0:ngen){ ## stats file includes generation 0
            myrow<-seq(d+g*(ndeme+1),dim(mystats)[1],(ndeme+1)*(ngen+1))

            meanmeannjunct<-c(meanmeannjunct,mean(as.numeric(mystats[myrow,11,k]),na.rm=T))
            sdmeannjunct<-c(sdmeannjunct,sd(as.numeric(mystats[myrow,11,k]),na.rm=T))
            meansdnjunct<-c(meansdnjunct,mean(as.numeric(mystats[myrow,12,k]),na.rm=T))
        }

        if(!(k-1)%%nruns){
            plot(0:ngen,rep(0.5,ngen+1),ylim=c(0,maxnjunct),axes=F,xlab="",
                 ylab="",type="n")

            box()
            axis(1,at=seq(0,ngen,10),padj=-0.4)
            axis(2,padj=0.6)
            if (d%%3==0){mtext("Generation",side=1,line=2,cex=1)}
            mtext("Junctions",side=2,line=2,cex=1)
            mtext(paste("Deme",d,sep=" "),side=3,line=0,cex=0.9)

            if(d%%3==0){
                legend("topleft",legend=c("mean njunct","+/- sd for mean njunct",
                                     "mean of sd njunct"),
                       col="darkgray", lwd=c(2,0.8,2), lty=c(1,1,2), bty="n")
            }
            if((d+2)%%3==0){
                legend("topleft",legend=runnames[k:(k+nruns-1)],col=runcols[1:nruns],
                       lwd=1.2, lty=1, bty="n")
            }
        }

        lines(0:ngen,meanmeannjunct,col=runcols[k],lty=1,lwd=2)
        lines(0:ngen,meanmeannjunct-sdmeannjunct,col=runcols[k],lty=1,lwd=0.8)
        lines(0:ngen,meanmeannjunct+sdmeannjunct,col=runcols[k],lty=1,lwd=0.8)
        lines(0:ngen,meansdnjunct,col=runcols[k],lty=2,lwd=2)

    }

}

dev.off()


